/*
 * Copyright 2020-2023 XLAB d.o.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

import { EvaluationTreeService } from "@/services/EvaluationTreeService";
import { scaleTree } from "@/mixins/scaleTree";
import { stringifyResources } from "@/helpers/helpers";
import router from "@/router";
import requirementCodeGuidelinesIdDictJSON from "@/assets/requirementCodeGuidelinesIdDict.json";

export default {
  state: {
    treeData: {},
    toeList: {},
    toeHistory: {},
    treeDataListByStateId: [],
    toms: [],
    requirementCodeGuidelinesIdDict: requirementCodeGuidelinesIdDictJSON,
  },
  mutations: {
    SAVE_TREE_DATA(state, treeData) {
      state.treeData = treeData;
    },
    SAVE_TOE_LIST(state, toeList) {
      state.toeList = toeList;
    },
    SAVE_TOE_HISTORY(state, toeHistory) {
      state.toeHistory = toeHistory;
    },
    SAVE_TREE_DATA_LIST(state, treeData) {
      state.treeDataListByStateId.push(treeData);
    },
    SAVE_TOMS_LIST(state, toms) {
      state.toms = toms;
    },
  },
  getters: {
    treeData: (state) => state.treeData,
    treeDataRoot: (state) => state.treeData?.root,
    toeList: (state) => state.toeList,
    toeHistory: (state) => state.toeHistory,
    treeByStateId: (state) => (stateId) => {
      return state.treeDataListByStateId.find(
        (tree) => tree.treeStateId === stateId,
      );
    },
    toms: (state) => state.toms,
    tomByCode: (state) => {
      if (!state.toms) {
        return {};
      }
      return state.toms?.reduce(function (map, item) {
        map[item.code] = item;
        return map;
      }, {});
    },
    requirementCodeGuidelinesIdDict: (state) =>
      state.requirementCodeGuidelinesIdDict,
  },
  actions: {
    async getTreeDataByStateId({ commit, getters }, stateId) {
      if (!stateId || stateId === getters.treeData.treeStateId) return;
      await router.replace({ query: { treeStateId: stateId } });
      if (stateId && getters.treeByStateId(stateId)) {
        return commit("SAVE_TREE_DATA", getters.treeByStateId(stateId));
      }

      await EvaluationTreeService.getTreeByStateId(stateId)
        .then((response) => {
          scaleTree.methods.collapseAllFromLevel(response.data.root, 1);
          response.data.root.treeStateId = response.data.treeStateId;
          stringifyResources(response.data);
          commit("SAVE_TREE_DATA", response.data);
          commit("SAVE_TREE_DATA_LIST", response.data);
        })
        .catch((err) => {
          console.error("Error getting tree by tree state id: " + err);
        });
    },
    async getToeList({ commit }) {
      await EvaluationTreeService.getToeList()
        .then((response) => {
          commit("SAVE_TOE_LIST", response.data);
        })
        .catch((err) => {
          console.error("Error getting ToE list: " + err);
        });
    },
    async getToeHistory({ commit }, id) {
      if (!id) {
        commit("SAVE_TOE_HISTORY", []);
        return;
      }
      await EvaluationTreeService.getToeHistory(id)
        .then((response) => {
          commit("SAVE_TOE_HISTORY", response.data);
        })
        .catch((err) => {
          console.error("Error getting ToE history: " + err);
        });
    },
    async initToeListAndTree({ dispatch, getters }, treeStateId) {
      await dispatch("getToeList").then(() => {
        if (treeStateId) {
          dispatch("getTreeDataByStateId", treeStateId).then(() => {
            dispatch("getToeHistory", getters.treeData.targetOfEvaluationId);
          });
        } else {
          dispatch("getToeHistory", getters.toeList[0]?.toeId).then(() => {
            dispatch(
              "getTreeDataByStateId",
              getters.toeHistory[0]?.treeStateId,
            );
          });
        }
      });
    },
    async selectToe({ dispatch, getters }, toeId) {
      await dispatch("getToeHistory", toeId).then(() => {
        dispatch("getTreeDataByStateId", getters.toeHistory[0].treeStateId);
      });
    },
    async getTomList({ commit, getters }) {
      if (getters.toms?.length) {
        return getters.toms;
      }
      await EvaluationTreeService.getToms()
        .then((response) => {
          commit("SAVE_TOMS_LIST", response.data);
        })
        .catch((err) => {
          console.error("Error getting TOMs list: " + err);
        });
      return getters.toms;
    },
    async getTomByCode({ commit, getters }, code) {
      if (getters.tomByCode[code]) {
        return getters.tomByCode[code];
      }
      await EvaluationTreeService.getToms()
        .then((response) => {
          commit("SAVE_TOMS_LIST", response.data);
        })
        .catch((err) => {
          console.error("Error getting TOMs list: " + err);
        });
      return getters.tomByCode[code];
    },
  },
};
